/*
Copyright (C) 2004-2023 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2023 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: progress dialog
//-----------------------------------------------------------------------------

#include "DialogCommandLineWizard.h"
#include "resource.h"

#define CMD_ARG_HEADER  TEXT("@rem use either\r\n") \
    TEXT("@rem .bat\r\n") \
    TEXT("@rem cmd /c ShortcutsSearchAndReplace64.exe args\r\n") \
    TEXT("@rem START /WAIT ShortcutsSearchAndReplace64.exe args\r\n\r\n")

#include "../tools/File/TextFile.h"

CDialogCommandLineWizard::CDialogCommandLineWizard(COutputString* pStrCmdLineReplace,COutputString* pStrCmdLineSearch,COutputString* pStrCmdLineSearchDead,
                                                    HINSTANCE hInstance,HWND hWndParentDialog,int DialogResourceId,int IconResourceId,int EditResourceId)
            :CDialogText(hInstance,hWndParentDialog,DialogResourceId,IconResourceId,EditResourceId)
{
    this->pStrCmdLineReplace = pStrCmdLineReplace;
    this->pStrCmdLineSearch = pStrCmdLineSearch;
    this->pStrCmdLineSearchDead = pStrCmdLineSearchDead;
}
CDialogCommandLineWizard::~CDialogCommandLineWizard()
{
    
}


BOOL CDialogCommandLineWizard::OnInit()
{
    this->SetDlgButtonIcon(IDOK,IDI_ICON_MENU_SAVE,16,16);

    ///////////////////////
    // ComboboxOperation
    ///////////////////////
    this->ComboboxOperation.CreateControlFromExistinghWnd(this->GetDlgItem(IDC_COMBO_COMMAND_LINE_WIZARD));
    this->ComboboxOperation.SetEventsHandler((IComboboxEvents*)(CComboboxEventsBase*)this);   
    // WARNING : MUST BE inserted in same order than ComboOperations
    this->ComboboxOperation.AddString(TEXT("Replace"));
    this->ComboboxOperation.AddString(TEXT("Search"));
    this->ComboboxOperation.AddString(TEXT("Search Dead Links"));

    this->ComboboxOperation.SetCurrentSelected(0);

    ///////////////////////
    // ComboboxPostOperation
    ///////////////////////
    this->ComboboxPostOperation.CreateControlFromExistinghWnd(this->GetDlgItem(IDC_COMBO_COMMAND_LINE_WIZARD_POST_OPERATION));
    this->ComboboxPostOperation.SetEventsHandler((IComboboxEvents*)(CComboboxEventsBase*)this);
    // WARNING : MUST BE inserted in same order than ComboPostSearchOperations
    this->ComboboxPostOperation.AddString(TEXT("None"));
    this->ComboboxPostOperation.AddString(TEXT("Resolve Found"));
    this->ComboboxPostOperation.AddString(TEXT("Delete Found"));

    this->ComboboxPostOperation.SetCurrentSelected(0);

    this->ShowPostOperation(FALSE);
    this->UpdateCmdLine();

    this->CDialogText::OnInit();

    return TRUE;
}

BOOL CDialogCommandLineWizard::OnClose()
{

    this->CDialogText::OnClose();
    return TRUE;
}

#define CDialogCommandLineWizard_DIALOG_MIN_WIDTH   300
#define CDialogCommandLineWizard_DIALOG_MIN_HEIGHT  200

void CDialogCommandLineWizard::OnSizing(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(wParam);
    RECT* pRect = (RECT*)lParam;
    // check min width and min height
    if ((pRect->right-pRect->left)<CDialogCommandLineWizard_DIALOG_MIN_WIDTH)
        pRect->right=pRect->left+CDialogCommandLineWizard_DIALOG_MIN_WIDTH;
    if ((pRect->bottom-pRect->top)<CDialogCommandLineWizard_DIALOG_MIN_HEIGHT)
        pRect->bottom=pRect->top+CDialogCommandLineWizard_DIALOG_MIN_HEIGHT;
}

void CDialogCommandLineWizard::Resize()
{
    RECT RectDlg;
    RECT Rect;
    POINT Point;
    this->GetInternalRect(&RectDlg);

    this->GetClientWindowRect(IDC_COMBO_COMMAND_LINE_WIZARD,&Rect);
    RectDlg.top = Rect.bottom + CDialogBase_SpaceBetweenControls;

    this->GetClientWindowRect(IDOK,&Rect);

    Point.x = RectDlg.right - (Rect.right-Rect.left) - CDialogBase_SpaceBetweenControls;
    Point.y = RectDlg.bottom - (Rect.bottom-Rect.top) - CDialogBase_SpaceBetweenControls;
    this->SetPositionFromId(IDOK,Point.x,Point.y);

    RectDlg.bottom = Point.y - CDialogBase_SpaceBetweenControls;

    this->SetRectFromId(this->EditResourceId,&RectDlg);
}

void CDialogCommandLineWizard::OnCommand(WPARAM wParam,LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);

    if (this->ComboboxOperation.OnCommand(wParam,lParam))
        return;

    if (this->ComboboxPostOperation.OnCommand(wParam,lParam))
        return;

    switch(LOWORD(wParam))
    {
    case IDOK:
        if (this->Save())
            this->Close(IDOK);
        break;
    case IDCANCEL:
        this->Close(IDCANCEL);
        break;
    }
}

void CDialogCommandLineWizard::OnComboBoxSelectionChanged(CCombobox* pSender,int SelectionIndex)
{
    UNREFERENCED_PARAMETER(SelectionIndex);

    if (pSender == &this->ComboboxOperation)
    {
        BOOL bShowPostOperation;
        switch(SelectionIndex)
        {
        default:
        case ComboOperations_Replace:
            bShowPostOperation = FALSE;
            break;
        case ComboOperations_Search:
        case ComboOperations_SearchDead:
            bShowPostOperation = TRUE;
            break;
        }
        this->ShowPostOperation(bShowPostOperation);
    }

    // if ( (pSender == &this->ComboboxOperation) || (pSender == &this->ComboboxPostOperation) )
        this->UpdateCmdLine();
}

void CDialogCommandLineWizard::UpdateCmdLine()
{
    switch(this->ComboboxOperation.GetCurrentSelected())
    {
    default:
    case ComboOperations_Replace:
        this->StrCmdLine.Copy(this->pStrCmdLineReplace);
        break;
    case ComboOperations_Search:
        this->StrCmdLine.Copy(this->pStrCmdLineSearch);
        this->AppendPostOperation(&this->StrCmdLine);
        break;
    case ComboOperations_SearchDead:
        this->StrCmdLine.Copy(this->pStrCmdLineSearchDead);
        this->AppendPostOperation(&this->StrCmdLine);
        break;
    }

    COutputString StrEditContent;
    // force "cmd /c " prefix for those which only do ctrl+c on the generated cmd line without generating the .bat file
    StrEditContent.Append(_DefineStringAndDefineStringSize(TEXT("cmd /c ")));
    StrEditContent.Append(&this->StrCmdLine);
    this->SetDlgItemTextT(this->EditResourceId,StrEditContent.GetString());
}

void CDialogCommandLineWizard::AppendPostOperation(IN OUT COutputString* pOutputString)
{
    // SearchPostOp=Resolve/Delete : Post search operation. Applies to Search or SearchDead Operation
    switch(this->ComboboxPostOperation.GetCurrentSelected())
    {
    default:
    case ComboPostSearchOperations_None:
        // Append nothing
        break;
    case ComboPostSearchOperations_Resolve:
        pOutputString->Append(_DefineStringAndDefineStringSize(TEXT(" SearchPostOp=Resolve")));
        break;
    case ComboPostSearchOperations_Delete:
        pOutputString->Append(_DefineStringAndDefineStringSize(TEXT(" SearchPostOp=Delete")));
        break;
    }
}

void CDialogCommandLineWizard::ShowPostOperation(BOOL bShowPostOperation)
{
    this->Show(IDC_STATIC_POST_OPERATION_COMMAND_LINE_WIZARD,bShowPostOperation);
    ::ShowWindow(this->ComboboxPostOperation.GetControlHandle(),bShowPostOperation);
}

BOOL CDialogCommandLineWizard::Save()
{
    TCHAR  FilePath[MAX_PATH];   
    TCHAR* FileExtension;
    TCHAR* FileFilters;

    *FilePath = 0;
    FileExtension = _T("bat");
    FileFilters = _T("bat\0*.bat\0All\0*.*\0");

    OPENFILENAME ofn;
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=this->hWndDialog;
    ofn.hInstance=this->hInstance;
    ofn.lpstrFilter=FileFilters;
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_NOREADONLYRETURN|OFN_OVERWRITEPROMPT;
    ofn.lpstrDefExt=FileExtension;
    ofn.lpstrFile=FilePath;
    ofn.nMaxFile=MAX_PATH;

    if (!::GetSaveFileName(&ofn))
    {
        // DWORD dw = ::CommDlgExtendedError();
        return FALSE;
    }

    HANDLE hFile;
    if (!CTextFile::CreateTextFile(FilePath,&hFile))
    {
        this->ReportError(TEXT("Error creating file"));
        return FALSE;
    }
    
    CTextFile::WriteText(hFile,_DefineStringAndDefineStringSize(CMD_ARG_HEADER));
    CTextFile::WriteText(hFile,this->StrCmdLine.GetString(),this->StrCmdLine.GetLength());
    CTextFile::CloseFile(hFile);

    return TRUE;
}
