/*
Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004-2016 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for tooltip control
//-----------------------------------------------------------------------------

#pragma once

// required lib : comctl32.lib
#pragma comment (lib,"comctl32")
// require manifest

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for xp os
#endif

#include <windows.h>
#include <commctrl.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)

class CToolTip
{
protected:
    HINSTANCE hInstance;
    HWND hWndParent;
    HWND hwndToolTip;
    void CommonConstructor(HINSTANCE hInstance,HWND ControlHandle,TCHAR* pszText,BOOL BalloonTooltip,BOOL bCenterTipOnControl,BOOL CloseButton,BOOL ParseLinks);
public:
    enum TOOLTIP_ICON
    {
        ICON_NONE=TTI_NONE,ICON_INFO=TTI_INFO,ICON_WARNING=TTI_WARNING,ICON_ERROR=TTI_ERROR
#if (_WIN32_WINNT >= 0x0600)
        ,ICON_INFO_LARGE=TTI_INFO_LARGE,ICON_WARNING_LARGE=TTI_WARNING_LARGE,ICON_ERROR_LARGE=TTI_ERROR_LARGE
#endif
    };
    ///////////////////////////////////////////////////////////////////////////
    // Notice : use LPSTR_TEXTCALLBACK for pszText to manage text by yourself in WndProc
    ///////////////////////////////////////////////////////////////////////////

    CToolTip(HINSTANCE hInstance,HWND ControlHandle,TCHAR* pszText,BOOL BalloonTooltip,BOOL bCenterTipOnControl);
    // Title : MSDN : "the string pointed to by lParam must not exceed 100 TCHARs in length, including the terminating NULL."
    CToolTip(HINSTANCE hInstance,HWND ControlHandle,TCHAR* pszText,TCHAR* Title,HICON hIcon,BOOL BalloonTooltip,BOOL bCenterTipOnControl);
    // if ParseLinks == TRUE, link should be inside text like TEXT("Some Text <A>http://www.xyz.com</A> some other text\r\nsome other text")
    // link click is notified by WM_NOTIFY / LPNMHDR->code == TTN_LINKCLICK
    // /!\ WARNING ParseLinks not compatible with LPSTR_TEXTCALLBACK !!!
    // /!\ WARNING CloseButton : even though TRUE visible only if SetTitleWithIconHandle has been called
    CToolTip(HINSTANCE hInstance,HWND ControlHandle,TCHAR* pszText,BOOL BalloonTooltip,BOOL bCenterTipOnControl,BOOL CloseButton,BOOL ParseLinks);
    ~CToolTip();
    HWND GetControlHandle();
    LRESULT SetText(TCHAR* pszText);
    // SetTitle MSDN : the string pointed to by lParam must not exceed 100 TCHARs in length, including the terminating NULL. 
    // Title : MSDN : "the string pointed to by lParam must not exceed 100 TCHARs in length, including the terminating NULL."
    LRESULT SetTitle(TCHAR* Title,TOOLTIP_ICON Icon);
    // SetTitleWithIconHandle 
    // Title : MSDN : "the string pointed to by lParam must not exceed 100 TCHARs in length, including the terminating NULL."
    // only for os > xp sp2, else no icon will display 
    LRESULT SetTitleWithIconHandle(TCHAR* Title,HICON Icon);
    LRESULT SetTextColor(COLORREF Color);
    LRESULT SetBackGroundColor(COLORREF Color);
    LRESULT Activate(BOOL bActivate);
    LRESULT SetMaxWidth(int Width);

    void HideToolTip();
    void ShowToolTip();
};
