#include "RegularExpression.h"
#include "../String/AnsiUnicodeConvert.h"

CRegularExpression::CRegularExpression(TCHAR* RegExpression)
{
    this->pRegExpr = NULL;
    *this->RegExprErrorMessage = 0;

    try
    {
        this->pRegExpr = new tregex(RegExpression);
    }
    catch (std::exception& e)
    {
        CAnsiUnicodeConvert::AnsiToTchar(e.what(),this->RegExprErrorMessage,_countof(this->RegExprErrorMessage));
    }
    catch(...)
    {
        _sntprintf(this->RegExprErrorMessage,_countof(this->RegExprErrorMessage),TEXT("Unknown regular expression exception"));
        this->RegExprErrorMessage[_countof(this->RegExprErrorMessage)-1]=0;
    }
}
CRegularExpression::~CRegularExpression()
{
    if (this->pRegExpr)
        delete this->pRegExpr;
}

BOOL CRegularExpression::IsRegExpressionValid()
{
    return (this->pRegExpr!=NULL);
}

BOOL CRegularExpression::DoesMatch(TCHAR* String)
{
    BOOL bMatch=FALSE;
    this->DoesMatchEx(String,&bMatch,NULL, 0);
    return bMatch;
}

BOOL CRegularExpression::DoesMatchEx(TCHAR* String,OUT BOOL* bMatch,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    *bMatch = FALSE;
    if (this->pRegExpr == NULL)
    {
        if (ErrorMessage)
        {
            _tcsncpy(ErrorMessage,this->RegExprErrorMessage,ErrorMessageMaxLen);
            ErrorMessage[ErrorMessageMaxLen-1] = 0;
        }
        return FALSE;
    }

    try
    {
        *bMatch = (REGEX_NAMESPACE::regex_match(String, *this->pRegExpr)==true);
        return TRUE;
    }
    catch (std::exception& e)
    {
        if (ErrorMessage)
            CAnsiUnicodeConvert::AnsiToTchar(e.what(),ErrorMessage,ErrorMessageMaxLen);
    }
    catch(...)
    {
        if (ErrorMessage)
        {
            _sntprintf(ErrorMessage,ErrorMessageMaxLen,TEXT("Unknown regular expression exception"));
            ErrorMessage[ErrorMessageMaxLen-1]=0;
        }
    }
    return FALSE;
}

BOOL CRegularExpression::DoesMatch(TCHAR* String,TCHAR* RegExpression)
{
    BOOL bMatch=FALSE;
    DoesMatchEx(String,RegExpression,&bMatch,NULL, 0);
    return bMatch;
}

BOOL CRegularExpression::DoesMatchEx(TCHAR* String,TCHAR* RegExpression,OUT BOOL* bMatch,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    CRegularExpression RegularExpression(RegExpression);
    return RegularExpression.DoesMatchEx(String,bMatch,ErrorMessage,ErrorMessageMaxLen);
}

tstring CRegularExpression::Replace(TCHAR* String,TCHAR* StrReplace)
{
    return this->ReplaceEx(String,StrReplace,NULL,0);
}
tstring CRegularExpression::ReplaceEx(TCHAR* String,TCHAR* StrReplace,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    tstring StdString = String;
    tstring StdStrReplace = StrReplace;

    return this->ReplaceEx(StdString,StdStrReplace,ErrorMessage, ErrorMessageMaxLen);
}

tstring CRegularExpression::Replace(tstring String,tstring StrReplace)
{
    return this->ReplaceEx(String,StrReplace,NULL,0);
}
tstring CRegularExpression::ReplaceEx(tstring String,tstring StrReplace,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    // $n or $nn (2 number max) 
    //      n-th backreference (i.e., a copy of the n-th matched group specified with parentheses in the regex pattern).
    //      n must be an integer value designating a valid backreference, greater than 0, and of TWO DIGITS AT MOST.
    //      --> if number (like 9) follows group (1) in string use $019 instead of $19
    // $0 and $& the entire match
    // $` The prefix (i.e., the part of the target sequence that precedes the match)
    // $' The suffix (i.e., the part of the target sequence that follows the match)
    // $$ A single $ character.

    // $1 contains, 1nd capturing group
    // tstring strReplace = "toto-group1=$1";

    tstring StrOut;

    // empty output error string
    if (ErrorMessage)
        *ErrorMessage = 0;

    // check current reg ex validity
    if (this->pRegExpr == NULL)
    {
        if (ErrorMessage)
        {
            _tcsncpy(ErrorMessage,this->RegExprErrorMessage,ErrorMessageMaxLen);
            ErrorMessage[ErrorMessageMaxLen-1] = 0;
        }
        return StrOut;
    }

    // do the replace
    try
    {
        StrOut = REGEX_NAMESPACE::regex_replace(String, *this->pRegExpr , StrReplace); 
    }
    catch (std::exception& e)
    {
        if (ErrorMessage)
            CAnsiUnicodeConvert::AnsiToTchar(e.what(),ErrorMessage,ErrorMessageMaxLen);
    }
    catch(...)
    {
        if (ErrorMessage)
        {
            _sntprintf(ErrorMessage,ErrorMessageMaxLen,TEXT("Unknown regular expression exception"));
            ErrorMessage[ErrorMessageMaxLen-1]=0;
        }
    }

    return StrOut;
}

tstring CRegularExpression::Replace(TCHAR* String,TCHAR* RegExpression,TCHAR* StrReplace)
{
    return CRegularExpression::ReplaceEx(String,RegExpression,StrReplace,NULL,0);
}

tstring CRegularExpression::ReplaceEx(TCHAR* String,TCHAR* RegExpression,TCHAR* StrReplace,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    tstring StdString = String;
    tstring StdRegExpression = RegExpression;
    tstring StdStrReplace = StrReplace;
    
    return CRegularExpression::ReplaceEx(StdString,StdRegExpression,StdStrReplace,ErrorMessage, ErrorMessageMaxLen);
}

tstring CRegularExpression::Replace(tstring String,tstring RegExpression,tstring StrReplace)
{
    return CRegularExpression::ReplaceEx(String,RegExpression,StrReplace,NULL,0);
}

tstring CRegularExpression::ReplaceEx(tstring String,tstring RegExpression,tstring StrReplace,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    // $n or $nn (2 number max) 
    //      n-th backreference (i.e., a copy of the n-th matched group specified with parentheses in the regex pattern).
    //      n must be an integer value designating a valid backreference, greater than 0, and of TWO DIGITS AT MOST.
    //      --> if number (like 9) follows group (1) in string use $019 instead of $19
    // $0 and $& the entire match
    // $` The prefix (i.e., the part of the target sequence that precedes the match)
    // $ The suffix (i.e., the part of the target sequence that follows the match)
    // $$ A single $ character.

    // $1 contains, 1nd capturing group
    // tstring strReplace = "toto-group1=$1";
    CRegularExpression RegularExpression((TCHAR*)RegExpression.c_str());
    return RegularExpression.ReplaceEx(String,StrReplace,ErrorMessage,ErrorMessageMaxLen);
}

BOOL CRegularExpression::MultipleMatch(TCHAR* Content,pfCallBackMatchFound CallBackFunction,PVOID UserParam)
{
    return this->MultipleMatchEx(Content,CallBackFunction,UserParam,NULL,0);
}

BOOL CRegularExpression::MultipleMatchEx(TCHAR* Content,pfCallBackMatchFound CallBackFunction,PVOID UserParam,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    tstring StdContent = Content;
    return this->MultipleMatchEx(StdContent, CallBackFunction, UserParam, ErrorMessage, ErrorMessageMaxLen);
}

BOOL CRegularExpression::MultipleMatch(tstring Content,pfCallBackMatchFound CallBackFunction,PVOID UserParam)
{
    return this->MultipleMatchEx(Content,CallBackFunction,UserParam,NULL,0);
}


BOOL CRegularExpression::MultipleMatchEx(tstring Content,pfCallBackMatchFound CallBackFunction,PVOID UserParam,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    
/*
    std::regex RegularEpression("ab(c)");
    tstring Content = "abcdefabcghiabc";
    std::smatch SingleMatch;
    
    uint32_t Cnt;
    while (std::regex_search( Content, SingleMatch, RegularEpression)) 
    {
        std::cout << Cnt << ": " << SingleMatch[0] << std::endl;
        
        std::cout << "Match value: " << SingleMatch.str() << " at Position " << SingleMatch.position() << "length " << SingleMatch.length() << '\n';
        std::cout << "    Capture: " << SingleMatch[1].str() << " at Position " << SingleMatch.position(1) << "length " << SingleMatch.length(1) << '\n';
        Cnt += 1;
        Content = SingleMatch.suffix().str();
    }     
    

    for(std::sregex_iterator i = std::sregex_iterator(Content.begin(), Content.end(), RegularEpression);
                             i != std::sregex_iterator();
                             ++i)
    {
        SingleMatch = *i;
        // SingleMatch.position(n) : n = 0 full expression start
        //                 n > 0 single group start
        std::cout << "Match value: " << SingleMatch.str() << " at Position " << SingleMatch.position() << "length " << SingleMatch.length() << '\n';
        std::cout << "    Capture: " << SingleMatch[1].str() << " at Position " << SingleMatch.position(1) << "length " << SingleMatch.length(1) << '\n';
    }
*/

    if (this->pRegExpr == NULL)
    {
        if (ErrorMessage)
        {
            _tcsncpy(ErrorMessage,this->RegExprErrorMessage,ErrorMessageMaxLen);
            ErrorMessage[ErrorMessageMaxLen-1] = 0;
        }
        return FALSE;
    }

    try
    {
        REGEX_NAMESPACE::tsmatch SingleMatch;
        for(REGEX_NAMESPACE::tsregex_iterator i = REGEX_NAMESPACE::tsregex_iterator(Content.begin(), Content.end(), *this->pRegExpr);
            i != REGEX_NAMESPACE::tsregex_iterator();
            ++i)
        {
            SingleMatch = *i;

            if (CallBackFunction(&SingleMatch,UserParam) != TRUE)
                break;
        
            // SingleMatch.position(n) : n = 0 full expression start
            //                           n > 0 single group start
            // std::cout << "Match value: " << SingleMatch.str() << " at Position " << SingleMatch.position() << "length " << SingleMatch.length() << '\n';
            // std::cout << "    Capture: " << SingleMatch[1].str() << " at Position " << SingleMatch.position(1) << "length " << SingleMatch.length(1) << '\n';
        }

        return TRUE;
    }
    catch (std::exception& e)
    {
        if (ErrorMessage)
            CAnsiUnicodeConvert::AnsiToTchar(e.what(),ErrorMessage,ErrorMessageMaxLen);
    }
    catch(...)
    {
        if (ErrorMessage)
        {
            _sntprintf(ErrorMessage,ErrorMessageMaxLen,TEXT("Unknown regular expression exception"));
            ErrorMessage[ErrorMessageMaxLen-1]=0;
        }
    }

    return FALSE;
}

BOOL CRegularExpression::MultipleMatch(TCHAR* Content,TCHAR* RegExpression,pfCallBackMatchFound CallBackFunction,PVOID UserParam)
{
    return CRegularExpression::MultipleMatchEx(Content,RegExpression,CallBackFunction,UserParam,NULL,0);
}

BOOL CRegularExpression::MultipleMatchEx(TCHAR* Content,TCHAR* RegExpression,pfCallBackMatchFound CallBackFunction,PVOID UserParam,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    tstring StdContent = Content;
    tstring StdRegExpression = RegExpression;
    return CRegularExpression::MultipleMatchEx(StdContent,StdRegExpression,CallBackFunction,UserParam,ErrorMessage,ErrorMessageMaxLen);
}

BOOL CRegularExpression::MultipleMatch(tstring Content,tstring RegExpression,pfCallBackMatchFound CallBackFunction,PVOID UserParam)
{
    return CRegularExpression::MultipleMatchEx(Content,RegExpression,CallBackFunction,UserParam,NULL,0);
}

BOOL CRegularExpression::MultipleMatchEx(tstring Content,tstring RegExpression,pfCallBackMatchFound CallBackFunction,PVOID UserParam,OUT TCHAR* ErrorMessage, IN SIZE_T ErrorMessageMaxLen)
{
    CRegularExpression RegularExpression((TCHAR*)RegExpression.c_str());
    return RegularExpression.MultipleMatchEx(Content,CallBackFunction,UserParam,ErrorMessage,ErrorMessageMaxLen);
}
